# -*- coding: utf-8 -*-
# __author__ = 'XingHuan'
# 9/5/2017

try:
    from PySide.QtCore import *
    from PySide.QtGui import *
except:
    from PySide2.QtCore import *
    from PySide2.QtGui import *
    from PySide2.QtWidgets import *
import os
import sys

import batchKnob_path as BP
import batchKnob_help as BH
import batchKnob_nuke as BN


inNuke = BP.inNuke
Global_Folder = BP.GLOBAL_Folder
Icons_Folder = "%s/icons" % Global_Folder
Current_User = BH.Current_User


VER = "v1.0.916"
AUTHOR = "XingHuan"
DATE = "2017/9/16"


class BatchKnob(QWidget):
    def __init__(self):
        super(BatchKnob, self).__init__()

        #if not inNuke:
        #    self.setStyleSheet("background-color: rgb(50, 50, 50)")
        self.setWindowTitle("BatchKnob")
        self.setWindowFlags(Qt.WindowStaysOnTopHint)

        self.topLayout = QHBoxLayout()
        self.addNodeButton = QLabelButton("add")
        self.addNodeButton.setToolTip("add nodes")
        self.connect(self.addNodeButton, SIGNAL("buttonClicked()"), self.add_nodes)
        self.clearNodeButton = QLabelButton("clear")
        self.clearNodeButton.setToolTip("clear nodes")
        self.connect(self.clearNodeButton, SIGNAL("buttonClicked()"), self.clear_nodes)
        self.refreshNodeButton = QLabelButton("refresh")
        self.connect(self.refreshNodeButton, SIGNAL("buttonClicked()"), self.refresh_nodes)
        self.topLayout.addWidget(self.addNodeButton)
        self.topLayout.addWidget(self.clearNodeButton)
        self.topLayout.addWidget(self.refreshNodeButton)
        self.topLayout.addStretch()


        self.globalKnobLayout = QHBoxLayout()
        self.globalKnobNameLabel = QLabel("knob:")
        self.globalKnobName = QComboBox()
        self.globalKnobName.currentIndexChanged.connect(self.change_global_knob)
        self.globalHasExpression = QCheckBox("expression")
        self.globalHasExpression.clicked.connect(self.change_global_hasExpression)
        self.globalHasReplace = QCheckBox("replace")
        self.globalHasReplace.clicked.connect(self.change_global_hasReplace)
        self.globalReplaceFromEdit = QLineEdit()
        self.globalReplaceFromEdit.setFixedWidth(70)
        self.globalReplaceFromEdit.textChanged.connect(self.change_global_replaceFrom)
        self.globalReplaceToLabel = QLabel("to")
        self.globalReplaceToEdit = QLineEdit()
        self.globalReplaceToEdit.setFixedWidth(70)
        self.globalReplaceToEdit.textChanged.connect(self.change_global_replaceTo)
        self.globalKnobValueAfterEdit1 = QLineEdit()
        self.globalKnobValueAfterEdit1.textChanged.connect(self.change_global_valueAfter)
        self.globalKnobValueAfterEdit2 = QTextEdit()
        self.globalKnobValueAfterEdit2.textChanged.connect(self.change_global_valueAfter)
        self.globalKnobValueAfterEdit2.setMaximumHeight(45)
        self.globalKnobValueAfterEditSwitch = QLabelButton("multiline")
        self.connect(self.globalKnobValueAfterEditSwitch, SIGNAL("buttonClicked()"), self.switch_globalKnobValueAfterEdit)
        self.globalKnobLayout.addWidget(self.globalKnobNameLabel)
        self.globalKnobLayout.addWidget(self.globalKnobName)
        self.globalKnobLayout.addSpacing(20)
        self.globalKnobLayout.addWidget(self.globalHasExpression)
        self.globalKnobLayout.addSpacing(20)
        self.globalKnobLayout.addWidget(self.globalHasReplace)
        self.globalKnobLayout.addWidget(self.globalReplaceFromEdit)
        self.globalKnobLayout.addWidget(self.globalReplaceToLabel)
        self.globalKnobLayout.addWidget(self.globalReplaceToEdit)
        self.globalKnobLayout.addSpacing(20)
        self.globalKnobLayout.addWidget(self.globalKnobValueAfterEdit1)
        self.globalKnobLayout.addWidget(self.globalKnobValueAfterEdit2)
        self.globalKnobLayout.addWidget(self.globalKnobValueAfterEditSwitch)
        self.globalKnobValueAfterSwitch = 0  # edit1



        self.nodeScrollArea = QScrollArea()
        self.nodesWidget = NodesWidget()
        self.connect(self.nodesWidget, SIGNAL("addSomeNodes()"), self.refresh_knobs_list)
        self.connect(self.nodesWidget, SIGNAL("removeOneNode()"), self.refresh_knobs_list)
        self.connect(self.nodesWidget, SIGNAL("clearAllNodes()"), self.refresh_knobs_list)
        self.nodeScrollArea.setWidget(self.nodesWidget)
        self.nodeScrollArea.setWidgetResizable(True)



        self.bottomLayout = QHBoxLayout()
        self.aboutButton = QLabelButton("about")
        self.connect(self.aboutButton, SIGNAL("buttonClicked()"), self.show_about)
        self.prefButton = QLabelButton("pref")
        self.connect(self.prefButton, SIGNAL("buttonClicked()"), self.show_pref)
        self.okButton = QPushButton("apply")
        self.okButton.setStyleSheet("color:rgb(180, 180, 180); background-color:rgb(39, 92, 132)")
        self.okButton.clicked.connect(self.apply_to_knobs)
        self.cancelButton = QPushButton("colse")
        self.cancelButton.clicked.connect(self.close_widget)
        self.bottomLayout.addStretch()
        self.bottomLayout.addWidget(self.aboutButton)
        self.bottomLayout.addWidget(self.prefButton)
        self.bottomLayout.addWidget(self.okButton)
        self.bottomLayout.addWidget(self.cancelButton)




        self.masterLayout = QVBoxLayout()
        self.masterLayout.addLayout(self.topLayout)
        self.masterLayout.addLayout(self.globalKnobLayout)
        self.masterLayout.addWidget(self.nodeScrollArea)
        #self.masterLayout.addStretch()
        self.masterLayout.addLayout(self.bottomLayout)
        self.setLayout(self.masterLayout)

        self.prefWindow = KnobPrefEditWidget()

        self.resize(800, 300)
        self.reset()


    def reset(self):
        self.globalHasExpression.setChecked(False)
        self.globalHasReplace.setChecked(False)
        self.globalReplaceFromEdit.setEnabled(False)
        self.globalReplaceFromEdit.setText("")
        self.globalReplaceToEdit.setEnabled(False)
        self.globalReplaceToEdit.setText("")
        self.globalKnobValueAfterEdit1.setEnabled(True)
        self.globalKnobValueAfterEdit1.setText("")
        self.globalKnobValueAfterEdit2.setEnabled(True)
        self.globalKnobValueAfterEdit2.setText("")
        self.globalKnobValueAfterSwitch = 0 # edit1
        self.globalKnobValueAfterEdit1.setHidden(self.globalKnobValueAfterSwitch)
        self.globalKnobValueAfterEdit2.setHidden(1 - self.globalKnobValueAfterSwitch)
        self.nodesWidget.clear_nodes()

    def switch_globalKnobValueAfterEdit(self):
        self.globalKnobValueAfterSwitch = 1 - self.globalKnobValueAfterSwitch
        self.globalKnobValueAfterEdit1.setHidden(self.globalKnobValueAfterSwitch)
        self.globalKnobValueAfterEdit2.setHidden(1 - self.globalKnobValueAfterSwitch)
        self.change_global_editSwitch()

    def add_nodes(self):
        #print "add nodes"
        self.nodesWidget.show_add_nodes_widget()


    def clear_nodes(self):
        #print "clear nodes"
        self.nodesWidget.clear_nodes()

    def refresh_knobs_list(self):
        #print "refresh knobs list"
        self.nodesWidget.refresh_knobs_name_list()
        newKnobsList = self.nodesWidget.knobNames
        nodeClasses = self.nodesWidget.nodeClasses
        if len(nodeClasses) == 1:
            newKnobsListSorted = BH.sort_knobs(nodeClasses[0], newKnobsList)
        else:
            newKnobsListSorted = BH.sort_knobs("All", newKnobsList)
        self.globalKnobName.currentIndexChanged.disconnect(self.change_global_knob)
        lastCount = self.globalKnobName.count()
        if lastCount > 0:
            for i in range(lastCount):
                self.globalKnobName.removeItem(0)
        if len(newKnobsListSorted) > 0:
            self.globalKnobName.addItem("")
            self.globalKnobName.addItems(newKnobsListSorted)
        self.update()
        self.globalKnobName.currentIndexChanged.connect(self.change_global_knob)

    def change_global_knob(self):
        #print "change global knob"
        currentKnob = self.globalKnobName.currentText()
        print currentKnob
        for nodeWidget in self.nodesWidget.nodeWidgets:
            nodeWidget.change_index_from_knob(currentKnob)

    def change_global_hasExpression(self):
        #print "change global hasExpression"
        currentHasExpression = self.globalHasExpression.isChecked()
        for nodeWidget in self.nodesWidget.nodeWidgets:
            nodeWidget.change_hasExp_from_global(currentHasExpression)

    def change_global_editSwitch(self):
        #print "change global editSwitch"
        currentEditSwitch = self.globalKnobValueAfterSwitch
        for nodeWidget in self.nodesWidget.nodeWidgets:
            nodeWidget.change_edieSwitch_from_global(currentEditSwitch)


    def change_global_hasReplace(self):
        #print "change global hasReplace"
        currentHasReplace = self.globalHasReplace.isChecked()
        self.globalReplaceFromEdit.setEnabled(currentHasReplace)
        self.globalReplaceToEdit.setEnabled(currentHasReplace)
        self.globalKnobValueAfterEdit1.setEnabled(1 - currentHasReplace)
        self.globalKnobValueAfterEdit2.setEnabled(1 - currentHasReplace)
        self.change_valueAfter()

    def change_global_replaceFrom(self):
        #print "change global replaceFrom"
        self.change_valueAfter()

    def change_global_replaceTo(self):
        #print "change global replaceTo"
        self.change_valueAfter()

    def change_global_valueAfter(self):
        #print "change global valueAfter"
        self.change_valueAfter()

    def change_valueAfter(self):
        #print "change valueAfter"
        currentHasReplace = self.globalHasReplace.isChecked()
        currentReplaceFrom = self.globalReplaceFromEdit.text()
        currentReplaceTo = self.globalReplaceToEdit.text()
        if self.globalKnobValueAfterSwitch == 0:
            currentValueAfter = self.globalKnobValueAfterEdit1.text()
        else:
            currentValueAfter = self.globalKnobValueAfterEdit2.toPlainText()
        for nodeWidget in self.nodesWidget.nodeWidgets:
            nodeWidget.change_valueAfter_from_global(currentHasReplace, currentReplaceFrom, currentReplaceTo,
                                                     currentValueAfter)

    def apply_to_knobs(self):
        #print "apply"
        for nodeWidget in self.nodesWidget.nodeWidgets:
            knobName = nodeWidget.knobName.currentText()
            hasExpression = nodeWidget.hasExpression.isChecked()
            editSwitch = nodeWidget.knobValueAfterSwitch
            valueAfter1 = nodeWidget.knobValueAfterEdit1.text()
            valueAfter2 = nodeWidget.knobValueAfterEdit2.toPlainText()
            if editSwitch == 0:
                finalValueStr = valueAfter1
            else:
                finalValueStr = valueAfter2
            if not hasExpression:
                if nodeWidget.node.knob(knobName).hasExpression():
                    c = nodeWidget.node.knob(knobName).animation(0)
                    nodeWidget.node.knob(knobName).deleteAnimation(c)
                valueBefore = nodeWidget.node.knob(knobName).value()
                finalValue = BH.convert_value(valueBefore, finalValueStr)
                nodeWidget.node.knob(knobName).setValue(finalValue)
            else:
                nodeWidget.node.knob(knobName).setExpression(finalValueStr)
        #self.close_widget()
        self.refresh_nodes()


    def close_widget(self):
        #print "close"
        self.close()

    def show_about(self):
        #print self.width()
        #print self.height()
        self.aboutWidget = AboutWidget()
        self.aboutWidget.show()

    def show_pref(self):
        self.prefWindow.showWindow()

    def refresh_nodes(self):
        self.nodesWidget.refresh_knobs_value()


class NodeWidget(QWidget, QObject):
    def __init__(self, node):
        super(NodeWidget, self).__init__()

        self.node = node

        self.setObjectName(self.node.name())

        self.masterLayout = QHBoxLayout()
        self.masterLayout.setAlignment(Qt.AlignCenter)
        self.nodeName = QLabel("%s" % self.node.name())
        self.nodeName.setFixedWidth(120)
        self.nodeName.setAlignment(Qt.AlignCenter)
        self.knobName = QComboBox()
        self.knobNameList = self.node.knobs().keys()
        self.knobNameListSorted = BH.sort_knobs(self.node.Class(), self.knobNameList)
        self.knobName.setFixedWidth(80)
        self.knobName.addItems(self.knobNameListSorted)
        self.knobName.setCurrentIndex(0)
        self.knobName.currentIndexChanged.connect(self.update_knob_value)
        self.knobValueBeforeEdit1 = QLineEdit()
        self.knobValueBeforeEdit1.setEnabled(False)
        self.knobValueBeforeEdit2 = QTextEdit()
        self.knobValueBeforeEdit2.setEnabled(False)
        self.knobValueBeforeEdit2.setMaximumHeight(45)
        self.knobValueBeforeEdit2.setHidden(True)
        self.hasExpression = QCheckBox("expression")
        self.knobValueAfterEdit1 = QLineEdit()
        self.knobValueAfterEdit2 = QTextEdit()
        self.knobValueAfterEdit2.setMaximumHeight(45)
        self.knobValueAfterEdit2.setHidden(True)
        self.knobValueAfterEditSwitch = QLabelButton("multiline")
        self.connect(self.knobValueAfterEditSwitch, SIGNAL("buttonClicked()"),self.switch_knobValueAfterEdit)
        self.knobValueAfterSwitch = 0  # edit1
        self.removeLabelButton = QLabelButton("clear")
        self.removeLabelButton.setToolTip("remove")
        self.connect(self.removeLabelButton, SIGNAL("buttonClicked()"), self.remove_this_node)


        self.masterLayout.addWidget(self.nodeName)
        self.masterLayout.addWidget(self.knobName)
        self.masterLayout.addWidget(self.knobValueBeforeEdit1)
        self.masterLayout.addWidget(self.knobValueBeforeEdit2)
        self.masterLayout.addSpacing(20)
        self.masterLayout.addWidget(self.hasExpression)
        self.masterLayout.addWidget(self.knobValueAfterEdit1)
        self.masterLayout.addWidget(self.knobValueAfterEdit2)
        self.masterLayout.addWidget(self.knobValueAfterEditSwitch)
        self.masterLayout.addWidget(self.removeLabelButton)

        self.setLayout(self.masterLayout)

        self.update_knob_value()


    def update_knob_value(self):
        currentKnob = self.knobName.currentText()
        #print currentKnob
        hasExpression = self.node.knob(currentKnob).hasExpression()
        if hasExpression:
            currentValue = self.node.knob(currentKnob).animation(0).expression()
        else:
            currentValue = str(self.node.knob(currentKnob).value())
        self.knobValueBeforeEdit1.setText(currentValue)
        self.knobValueBeforeEdit2.setText(currentValue)
        self.hasExpression.setChecked(hasExpression)
        self.knobValueAfterEdit1.setText(currentValue)
        self.knobValueAfterEdit2.setText(currentValue)
        if currentValue.find("\n") != -1:
            self.knobValueAfterSwitch = 0
            self.switch_knobValueAfterEdit()
        else:
            self.knobValueAfterSwitch = 1
            self.switch_knobValueAfterEdit()


    def switch_knobValueAfterEdit(self):
        self.knobValueAfterSwitch = 1 - self.knobValueAfterSwitch
        self.knobValueBeforeEdit1.setHidden(self.knobValueAfterSwitch)
        self.knobValueBeforeEdit2.setHidden(1 - self.knobValueAfterSwitch)
        self.knobValueAfterEdit1.setHidden(self.knobValueAfterSwitch)
        self.knobValueAfterEdit2.setHidden(1 - self.knobValueAfterSwitch)

    def remove_this_node(self):
        # print "remove %s" % self.objectName()
        self.emit(SIGNAL("removeThisNode(QObject)"), self)


    def change_index_from_knob(self, knobName):
        #print "change_index_from_knob"
        if knobName != "":
            for i in range(self.knobName.count()):
                if self.knobName.itemText(i) == knobName:
                    self.knobName.setCurrentIndex(i)

    def change_hasExp_from_global(self, hasExp):
        self.hasExpression.setChecked(hasExp)

    def change_edieSwitch_from_global(self, editSwitch):
        self.knobValueAfterSwitch = editSwitch
        self.knobValueBeforeEdit1.setHidden(self.knobValueAfterSwitch)
        self.knobValueBeforeEdit2.setHidden(1 - self.knobValueAfterSwitch)
        self.knobValueAfterEdit1.setHidden(self.knobValueAfterSwitch)
        self.knobValueAfterEdit2.setHidden(1 - self.knobValueAfterSwitch)


    def change_valueAfter_from_global(self, hasReplace, replaceFrom, replaceTo, valueAfter):
        if not hasReplace:
            if valueAfter != "":
                self.knobValueAfterEdit1.setText(valueAfter)
                self.knobValueAfterEdit2.setText(valueAfter)
            else:
                self.knobValueAfterEdit1.setText(self.knobValueBeforeEdit1.text())
                self.knobValueAfterEdit2.setText(self.knobValueBeforeEdit2.toPlainText())
        else:
            textBefore1 = self.knobValueBeforeEdit1.text()
            textAfter1 = textBefore1.replace(replaceFrom, replaceTo)
            self.knobValueAfterEdit1.setText(textAfter1)
            textBefore2 = self.knobValueBeforeEdit2.toPlainText()
            textAfter2 = textBefore2.replace(replaceFrom, replaceTo)
            self.knobValueAfterEdit2.setText(textAfter2)



class NodesWidget(QWidget):
    def __init__(self):
        super(NodesWidget, self).__init__()

        self.nodesLayout = QVBoxLayout()
        self.nodesLayout.setSpacing(0)
        self.nodesNum = 0
        self.nodeWidgets = []
        self.nodeNames = []
        self.nodeClasses = []
        self.knobNames = []

        self.setLayout(self.nodesLayout)


    def show_add_nodes_widget(self):
        selectedNodes = BN.get_selected_nodes()
        if len(selectedNodes) > 0:
            if len(BN.get_nodes_class(selectedNodes)) > 1:
                self.nodeFilterWidget = NodesFilterWidget(selectedNodes)
                self.nodeFilterWidget.show()
                self.connect(self.nodeFilterWidget, SIGNAL("hasAddNodes()"), self.add_nodes)
            else:
                self.nodeFilterWidget = NodesFilterWidget(selectedNodes)
                self.add_nodes()

    def add_nodes(self):
        addNodes = self.nodeFilterWidget.filterNodes
        addClasses = BN.get_nodes_class(addNodes)
        for addClass in addClasses:
            if addClass not in self.nodeClasses:
                self.nodeClasses.append(addClass)
        # print addNodes
        # print self.nodeClasses
        if len(addNodes) > 0:
            for node in addNodes:
                if node.name() not in self.nodeNames:
                    self.nodeNames.append(node.name())
                    addNodeWidget = NodeWidget(node)
                    # print addNode.objectName()
                    self.nodesNum = self.nodesNum + 1
                    self.nodesLayout.addWidget(addNodeWidget)
                    self.connect(addNodeWidget, SIGNAL("removeThisNode(QObject)"), self.remove_node)
                    self.nodeWidgets.append(addNodeWidget)
            self.emit(SIGNAL("addSomeNodes()"))

    def remove_node(self, nodeWidget):
        print "remove %s" % nodeWidget.objectName()
        self.nodeWidgets.remove(nodeWidget)
        nodeWidget.deleteLater()
        self.nodeNames.remove(nodeWidget.objectName())
        self.emit(SIGNAL("removeOneNode()"))


    def clear_nodes(self):
        #print self.nodeWidgets
        for i in range(len(self.nodeWidgets)):
            nodeWidget = self.nodeWidgets[0]
            #print nodeWidget
            self.nodeWidgets.pop(0)
            nodeWidget.deleteLater()
            self.nodesNum = self.nodesNum - 1
            self.nodeNames.remove(nodeWidget.objectName())
        #print self.nodeWidgets
        self.nodeClasses = []
        self.emit(SIGNAL("clearAllNodes()"))

    def refresh_knobs_name_list(self):
        self.knobNames = []
        self.nodeClasses = []
        for nodeWidget in self.nodeWidgets:
            #print nodeWidget.node.knobs().keys()
            for key in nodeWidget.node.knobs().keys():
                if key not in self.knobNames:
                    self.knobNames.append(key)
            if nodeWidget.node.Class() not in self.nodeClasses:
                self.nodeClasses.append(nodeWidget.node.Class())


        #print self.knobNames

    def refresh_knobs_value(self):
        for nodeWidget in self.nodeWidgets:
            nodeWidget.update_knob_value()




class QLabelButton(QLabel):
    def __init__(self, name):
        super(QLabelButton, self).__init__()

        self.setToolTip("<font color=#FFFFFF>%s</font>" % name)
        self.imageFile = '%s/%s.png' % (Icons_Folder, name)
        self.setPixmap(QPixmap(self.imageFile).scaled(24, 24, Qt.KeepAspectRatio, Qt.SmoothTransformation))
        self.setStyleSheet("background-color: rgb(40, 40, 40, 0)")

    def enterEvent(self, event):
        self.setStyleSheet("background-color: rgb(70, 70, 70)")

    def leaveEvent(self, event):
        self.setStyleSheet("background-color: rgb(40, 40, 40, 0)")

    def mousePressEvent(self, event):
        self.setStyleSheet("background-color: rgb(40, 40, 40)")

    def mouseReleaseEvent(self, event):
        self.emit(SIGNAL('buttonClicked()'))
        self.setStyleSheet("background-color: rgb(70, 70, 70)")


class AboutWidget(QWidget):
    def __init__(self):
        super(AboutWidget, self).__init__()
        self.setWindowFlags(Qt.WindowStaysOnTopHint)
        self.setFixedSize(270, 250)
        self.setWindowTitle("About")

        self.setStyleSheet("color: white; background-color: rgb(50, 50, 50)")
        aboutToolbox = QLabel()
        aboutToolbox.setPixmap(QPixmap("%s/logo.png" % Icons_Folder).scaled(250, 100, Qt.KeepAspectRatio, Qt.SmoothTransformation))
        aboutVer = QLabel(VER)
        aboutVer.setStyleSheet("font:20px Arial")
        aboutAuthor = QLabel("Author:%s" % AUTHOR)
        aboutAuthor.setAlignment(Qt.AlignRight)
        aboutAuthor.setStyleSheet("font:15px Arial")
        aboutDate = QLabel("Date:%s" % DATE)
        aboutDate.setAlignment(Qt.AlignRight)
        aboutDate.setStyleSheet("font:15px Arial")

        self.masterLayout = QVBoxLayout()
        self.masterLayout.addWidget(aboutToolbox)
        self.masterLayout.addWidget(aboutVer)
        self.masterLayout.addWidget(aboutAuthor)
        self.masterLayout.addWidget(aboutDate)
        self.setLayout(self.masterLayout)

        self.adjustSize()
        screenRes = QDesktopWidget().screenGeometry()
        #self.move(QPoint(screenRes.width()/2,screenRes.height()/2)-QPoint((self.width()/2),(self.height()/2)))
        self.move(QPoint(0, 0))



class NodesFilterWidget(QWidget):
    def __init__(self, selectedNodes):
        super(NodesFilterWidget, self).__init__()

        self.setWindowFlags(Qt.WindowStaysOnTopHint)
        self.setWindowTitle("Node Filter")

        self.selectedNodes = selectedNodes

        self.layout1 = QHBoxLayout()
        self.nodeClassLabel = QLabel("node class:")
        self.nodeClassEdit = QLineEdit("*")
        self.nodeClassFilter = QComboBox()
        self.nodeClasses = []
        self.nodeClasses.append("*")
        self.nodeClasses.extend(BN.get_nodes_class(self.selectedNodes))
        self.nodeClassFilter.addItems(self.nodeClasses)
        self.nodeClassFilter.currentIndexChanged.connect(self.change_filter)
        self.nodeClassFilterMode = QCheckBox("add")
        self.layout1.addWidget(self.nodeClassLabel)
        self.layout1.addWidget(self.nodeClassEdit)
        self.layout1.addWidget(self.nodeClassFilter)
        self.layout1.addWidget(self.nodeClassFilterMode)

        self.layout2 = QHBoxLayout()
        self.okButton = QPushButton("ok")
        self.okButton.clicked.connect(self.click_ok)
        self.cancelButton = QPushButton("cancel")
        self.cancelButton.clicked.connect(self.click_cancel)
        self.layout2.addStretch()
        self.layout2.addWidget(self.okButton)
        self.layout2.addWidget(self.cancelButton)

        self.masterLayout = QVBoxLayout()
        self.masterLayout.addLayout(self.layout1)
        self.masterLayout.addLayout(self.layout2)
        self.setLayout(self.masterLayout)

        self.filterClasses = self.nodeClasses
        self.filterNodes = self.selectedNodes

    def change_filter(self):
        print "change filter"
        currentChosen = self.nodeClassFilter.currentText()
        if currentChosen == "*":
            self.filterClasses = self.nodeClasses
            self.nodeClassEdit.setText("*")
        else:
            if self.nodeClassFilterMode.isChecked():
                if currentChosen not in self.filterClasses:
                    self.filterClasses.append(currentChosen)
                if "*" in self.filterClasses:
                    self.filterClasses.remove("*")
            else:
                self.filterClasses = [currentChosen]
            self.nodeClassEdit.setText(";".join(self.filterClasses))


    def click_ok(self):
        # print self.filterClasses
        self.filterNodes = []
        for node in self.selectedNodes:
            if BN.node_in_class(node, self.filterClasses):
                self.filterNodes.append(node)
        #print self.filterNodes
        self.emit(SIGNAL('hasAddNodes()'))
        self.close()

    def click_cancel(self):
        self.close()



class KnobPrefEditWidget(QWidget):
    def __init__(self):
        super(KnobPrefEditWidget, self).__init__()
        self.setWindowFlags(Qt.WindowStaysOnTopHint)
        self.setWindowTitle("Knob Pref")
        if not inNuke:
            self.setStyleSheet("color: white; background-color: rgb(50, 50, 50)")



        self.nodesLayout = QHBoxLayout()
        self.nodesButtonLayout = QVBoxLayout()
        self.nodesAddButton = QLabelButton("add")
        self.connect(self.nodesAddButton, SIGNAL('buttonClicked()'), self.add_node)
        self.nodesRemoveButton = QLabelButton("clear")
        self.nodesRemoveButton.setToolTip("remove")
        self.connect(self.nodesRemoveButton, SIGNAL('buttonClicked()'), self.del_nodes)
        self.nodesButtonLayout.addStretch()
        self.nodesButtonLayout.addWidget(self.nodesAddButton)
        self.nodesButtonLayout.addWidget(self.nodesRemoveButton)
        self.nodesButtonLayout.addStretch()
        self.nodesListLayout = QVBoxLayout()
        self.nodesListLabel = QLabel("nodes")
        self.nodesListLabel.setAlignment(Qt.AlignCenter)
        self.nodesListWidget = QListWidget()
        self.connect(self.nodesListWidget, SIGNAL('itemClicked(QListWidgetItem*)'), self.refresh_pref_knobs_list)
        self.connect(self.nodesListWidget, SIGNAL('itemClicked(QListWidgetItem*)'), self.refresh_all_knobs_list)
        self.nodesListLayout.addWidget(self.nodesListLabel)
        self.nodesListLayout.addWidget(self.nodesListWidget)
        self.nodesLayout.addLayout(self.nodesButtonLayout)
        self.nodesLayout.addLayout(self.nodesListLayout)

        self.prefKnobsLayout = QHBoxLayout()
        self.prefKnobsButtonLayout = QVBoxLayout()
        self.prefKnobsAddManualButton = QLabelButton("add")
        self.connect(self.prefKnobsAddManualButton, SIGNAL('buttonClicked()'), self.add_knob_manual)
        self.prefKnobsUpToTopButton = QLabelButton("up_to_top")
        self.connect(self.prefKnobsUpToTopButton, SIGNAL('buttonClicked()'), self.up_knob_to_top)
        self.prefKnobsUpButton = QLabelButton("up")
        self.connect(self.prefKnobsUpButton, SIGNAL('buttonClicked()'), self.up_knob)
        self.prefKnobsDownButton = QLabelButton("down")
        self.connect(self.prefKnobsDownButton, SIGNAL('buttonClicked()'), self.down_knob)
        self.prefKnobsDownToBottomButton = QLabelButton("down_to_bottom")
        self.connect(self.prefKnobsDownToBottomButton, SIGNAL('buttonClicked()'), self.down_knob_to_bottom)
        self.prefKnobsButtonLayout.addStretch()
        self.prefKnobsButtonLayout.addWidget(self.prefKnobsAddManualButton)
        self.prefKnobsButtonLayout.addWidget(self.prefKnobsUpToTopButton)
        self.prefKnobsButtonLayout.addWidget(self.prefKnobsUpButton)
        self.prefKnobsButtonLayout.addWidget(self.prefKnobsDownButton)
        self.prefKnobsButtonLayout.addWidget(self.prefKnobsDownToBottomButton)
        self.prefKnobsButtonLayout.addStretch()
        self.prefKnobsListLayout = QVBoxLayout()
        self.prefKnobsListLabel = QLabel("pref knobs")
        self.prefKnobsListLabel.setAlignment(Qt.AlignCenter)
        self.prefKnobsListWidget = QListWidget()
        self.prefKnobsListLayout.addWidget(self.prefKnobsListLabel)
        self.prefKnobsListLayout.addWidget(self.prefKnobsListWidget)
        self.prefKnobsLayout.addLayout(self.prefKnobsButtonLayout)
        self.prefKnobsLayout.addLayout(self.prefKnobsListLayout)

        self.allKnobsLayout = QHBoxLayout()
        self.allKnobsButtonLayout = QVBoxLayout()
        self.allKnobsLeftButton = QLabelButton("move_left")
        self.connect(self.allKnobsLeftButton, SIGNAL('buttonClicked()'), self.add_knobs)
        self.allKnobsRightButton = QLabelButton("move_right")
        self.connect(self.allKnobsRightButton, SIGNAL('buttonClicked()'), self.del_knobs)
        self.allKnobsButtonLayout.addStretch()
        self.allKnobsButtonLayout.addWidget(self.allKnobsLeftButton)
        self.allKnobsButtonLayout.addWidget(self.allKnobsRightButton)
        self.allKnobsButtonLayout.addStretch()
        self.allKnobsListLayout = QVBoxLayout()
        self.allKnobsListLabel = QLabel("all knobs")
        self.allKnobsListLabel.setAlignment(Qt.AlignCenter)
        self.allKnobsListSearch = QLineEdit()
        self.allKnobsListSearch.textChanged.connect(self.search_all_knobs_list)
        self.allKnobsListSearch.setStyleSheet(
            "color:rgb(220, 220, 220); padding: 2px 2px 2px 20px;background-image: url(%s/search.png);background-position: left;background-repeat: no-repeat" % Icons_Folder)
        self.allKnobsListWidget = QListWidget()
        self.allKnobsListWidget.setSelectionMode(QAbstractItemView.ExtendedSelection)
        self.allKnobsList = []
        self.allKnobsListLayout.addWidget(self.allKnobsListLabel)
        self.allKnobsListLayout.addWidget(self.allKnobsListSearch)
        self.allKnobsListLayout.addWidget(self.allKnobsListWidget)
        self.allKnobsLayout.addLayout(self.allKnobsButtonLayout)
        self.allKnobsLayout.addLayout(self.allKnobsListLayout)


        self.masterLayout = QHBoxLayout()
        self.masterLayout.addLayout(self.nodesLayout)
        self.masterLayout.addLayout(self.prefKnobsLayout)
        self.masterLayout.addLayout(self.allKnobsLayout)
        self.setLayout(self.masterLayout)

        self.resize(650, 450)
        self.refresh_nodes_list()

    def showWindow(self):
        self.refresh_nodes_list()
        self.refresh_pref_knobs_list()
        self.refresh_all_knobs_list()
        self.show()


    def get_selected_nodeName(self):
        selectedNodeName = ""
        if len(self.nodesListWidget.selectedItems())>0:
            selectedNodeItem = self.nodesListWidget.selectedItems()[0]
            selectedNodeName = selectedNodeItem.toolTip()
        return selectedNodeName

    def get_selected_prefKnobName(self):
        selectedPrefKnobName = ""
        if len(self.prefKnobsListWidget.selectedItems())>0:
            selectedPrefKnobItem = self.prefKnobsListWidget.selectedItems()[0]
            selectedPrefKnobName = selectedPrefKnobItem.toolTip()
        return selectedPrefKnobName

    def selected_pref_knob(self, knobName):
        for i in range(self.prefKnobsListWidget.count()):
            if self.prefKnobsListWidget.item(i).toolTip() == knobName:
                self.prefKnobsListWidget.setCurrentRow(i)

    def get_selected_allKnobNames(self):
        selectedAllKnobNames = []
        if len(self.allKnobsListWidget.selectedItems())>0:
            for i in self.allKnobsListWidget.selectedItems():
                selectedAllKnobName = i.toolTip()
                selectedAllKnobNames.append(selectedAllKnobName)
        return selectedAllKnobNames

    def refresh_nodes_list(self):
        print "refresh_nodes_list"
        nodes = BH.get_pref_nodes()
        nodes.sort()
        self.nodesListWidget.clear()
        for nodeName in nodes:
            i = QListWidgetItem(nodeName)
            i.setToolTip(nodeName)
            self.nodesListWidget.addItem(i)

    def refresh_pref_knobs_list(self):
        print "refresh_pref_knobs_list"
        selectedNodeName = self.get_selected_nodeName()
        if selectedNodeName != "":
            prefKnobNames = BH.get_pref_knobs_from_node(selectedNodeName)
            self.prefKnobsListWidget.clear()
            for prefKnobName in prefKnobNames:
                i = QListWidgetItem(prefKnobName)
                i.setToolTip(prefKnobName)
                self.prefKnobsListWidget.addItem(i)
        else:
            self.prefKnobsListWidget.clear()

    def refresh_all_knobs_list(self):
        print "refresh_all_knobs_list"
        selectedNodeName = self.get_selected_nodeName()
        if selectedNodeName != "":
            allKnobNames = BN.get_knobs_from_nodeName(selectedNodeName)
            self.allKnobsList = allKnobNames
            self.allKnobsListWidget.clear()
            for allKnobName in allKnobNames:
                i = QListWidgetItem(allKnobName)
                i.setToolTip(allKnobName)
                self.allKnobsListWidget.addItem(i)
        else:
            self.allKnobsListWidget.clear()

    def search_all_knobs_list(self):
        searchStr = self.allKnobsListSearch.text()
        self.allKnobsListWidget.clear()
        for allKnobName in self.allKnobsList:
            if allKnobName.find(searchStr) != -1:
                i = QListWidgetItem(allKnobName)
                i.setToolTip(allKnobName)
                self.allKnobsListWidget.addItem(i)


    def add_node(self):
        print "add_node"
        selectedNodeNames = BN.get_nodes_class(BN.get_selected_nodes())
        selectedNodeName = ""
        if len(selectedNodeNames) > 0:
            selectedNodeName = selectedNodeNames[0]
        nodeName = QInputDialog.getText(self, "add node", "node name", QLineEdit.Normal, selectedNodeName)[0]
        if nodeName != "":
            if nodeName not in BH.get_pref_nodes():
                BH.add_node(nodeName)
                self.refresh_nodes_list()

    def del_nodes(self):
        print "del_nodes"
        selectedNodeName = self.get_selected_nodeName()
        if selectedNodeName != "":
            BH.del_nodes([selectedNodeName])
            self.refresh_nodes_list()

    def add_knob_manual(self):
        print "add_knob_manual"
        selectedNodeName = self.get_selected_nodeName()
        if selectedNodeName != "":
            prefKnobName = QInputDialog.getText(self, "add pref knob", "knob name", QLineEdit.Normal, "")[0]
            if prefKnobName != "":
                BH.add_knobs(selectedNodeName, [prefKnobName])
                self.refresh_pref_knobs_list()

    def up_knob_to_top(self):
        self.move_pref_knob("up_to_top")
    def up_knob(self):
        self.move_pref_knob("up")
    def down_knob(self):
        self.move_pref_knob("down")
    def down_knob_to_bottom(self):
        self.move_pref_knob("down_to_bottom")

    def move_pref_knob(self, mode="up"):
        print "move %s" % mode
        selectedNodeName = self.get_selected_nodeName()
        selectedPrefKnobName = self.get_selected_prefKnobName()
        if selectedNodeName != "" and selectedPrefKnobName != "":
            if mode == "up_to_top":
                BH.up_knob_to_top(selectedNodeName, selectedPrefKnobName)
            if mode == "up":
                BH.up_knob(selectedNodeName, selectedPrefKnobName)
            if mode == "down":
                BH.down_knob(selectedNodeName, selectedPrefKnobName)
            if mode == "down_to_bottom":
                BH.down_knob_to_bottom(selectedNodeName, selectedPrefKnobName)
            self.refresh_pref_knobs_list()
            self.selected_pref_knob(selectedPrefKnobName)


    def add_knobs(self):
        print "add_knobs"
        selectedNodeName = self.get_selected_nodeName()
        selectedAllKnobName = self.get_selected_allKnobNames()
        if selectedNodeName != "" and selectedAllKnobName != []:
            BH.add_knobs(selectedNodeName, selectedAllKnobName)
            self.refresh_pref_knobs_list()

    def del_knobs(self):
        print "del_knobs"
        selectedNodeName = self.get_selected_nodeName()
        selectedPrefKnobName = self.get_selected_prefKnobName()
        if selectedNodeName != "" and selectedPrefKnobName != "":
            BH.del_knobs(selectedNodeName, [selectedPrefKnobName])
            self.refresh_pref_knobs_list()











if not inNuke:
    app = QApplication(sys.argv)
    test = BatchKnob()
    test.reset()
    #pyqtT = KnobPrefEditWidget()
    test.show()
    app.exec_()
else:
    batchKnob = BatchKnob()
    def main():
        batchKnob.reset()
        batchKnob.show()





